@@ -52,7 +52,7 @@ func NewLexerBytes(b []byte) *Lexer {
 	n := len(b)
 	if n == 0 {
 		z.buf = nullBuffer
-	} else if b[n-1] != 0 {
+	} else {
 		// Append NULL to buffer, but try to avoid reallocation
 		if cap(b) > n {
 			// Overwrite next byte but restore when done
