@@ -5,7 +5,6 @@ import (
 	"bytes"
 	"encoding/base64"
 	"errors"
-	"net/url"
 )
 
 // ErrBadDataURI is returned by DataURI when the byte slice does not start with 'data:' or is too short.
@@ -178,8 +177,8 @@ func DataURI(dataURI []byte) ([]byte, []byte, error) {
 							return nil, nil, err
 						}
 						data = decoded[:n]
-					} else if unescaped, err := url.QueryUnescape(string(data)); err == nil {
-						data = []byte(unescaped)
+					} else {
+						data = DecodeURL(data)
 					}
 					return mediatype, data, nil
 				}
@@ -190,6 +189,7 @@ func DataURI(dataURI []byte) ([]byte, []byte, error) {
 }
 
 // QuoteEntity parses the given byte slice and returns the quote that got matched (' or ") and its entity length.
+// TODO: deprecated
 func QuoteEntity(b []byte) (quote byte, n int) {
 	if len(b) < 5 || b[0] != '&' {
 		return 0, 0
@@ -221,9 +221,9 @@ func QuoteEntity(b []byte) (quote byte, n int) {
 			}
 		}
 	} else if len(b) >= 6 && b[5] == ';' {
-		if EqualFold(b[1:5], []byte{'q', 'u', 'o', 't'}) {
+		if bytes.Equal(b[1:5], []byte{'q', 'u', 'o', 't'}) {
 			return '"', 6 // &quot;
-		} else if EqualFold(b[1:5], []byte{'a', 'p', 'o', 's'}) {
+		} else if bytes.Equal(b[1:5], []byte{'a', 'p', 'o', 's'}) {
 			return '\'', 6 // &apos;
 		}
 	}
