@@ -1,4 +1,4 @@
-package css // import "github.com/tdewolff/parse/css"
+package css
 
 import (
 	"bytes"
@@ -71,7 +71,7 @@ func TestParse(t *testing.T) {
 		{false, "@media (max-width:400px)", "@media(max-width:400px);"},
 		{false, "@font-face { ; font:x; }", "@font-face{font:x;}"},
 		{false, "@-moz-font-face { ; font:x; }", "@-moz-font-face{font:x;}"},
-		{false, "@unknown abc { {} lala }", "@unknown abc{{}lala}"},
+		{false, "@unknown abc { {} lala }", "@unknown abc{{} lala }"},
 		{false, "a[x={}]{x:y;}", "a[x={}]{x:y;}"},
 		{false, "a[x=,]{x:y;}", "a[x=,]{x:y;}"},
 		{false, "a[x=+]{x:y;}", "a[x=+]{x:y;}"},
@@ -88,6 +88,8 @@ func TestParse(t *testing.T) {
 		{false, "@media{selector{", "@media{selector{"},
 
 		// bad grammar
+		{false, "}", "ERROR(})"},
+		{true, "}", "ERROR(})"},
 		{true, "~color:red", "ERROR(~color:red)"},
 		{true, "(color;red)", "ERROR((color;red))"},
 		{true, "color(;red)", "ERROR(color(;red))"},
@@ -118,7 +120,7 @@ func TestParse(t *testing.T) {
 				data = parse.Copy(data)
 				if grammar == ErrorGrammar {
 					if err := p.Err(); err != io.EOF {
-						data = append([]byte("ERROR("), data...)
+						data = []byte("ERROR(")
 						for _, val := range p.Values() {
 							data = append(data, val.Data...)
 						}
@@ -147,17 +149,13 @@ func TestParse(t *testing.T) {
 		})
 	}
 
-	test.T(t, ErrorGrammar.String(), "Error")
-	test.T(t, AtRuleGrammar.String(), "AtRule")
-	test.T(t, BeginAtRuleGrammar.String(), "BeginAtRule")
-	test.T(t, EndAtRuleGrammar.String(), "EndAtRule")
-	test.T(t, BeginRulesetGrammar.String(), "BeginRuleset")
-	test.T(t, EndRulesetGrammar.String(), "EndRuleset")
-	test.T(t, DeclarationGrammar.String(), "Declaration")
-	test.T(t, TokenGrammar.String(), "Token")
-	test.T(t, CommentGrammar.String(), "Comment")
-	test.T(t, CustomPropertyGrammar.String(), "CustomProperty")
-	test.T(t, GrammarType(100).String(), "Invalid(100)")
+	// coverage
+	for i := 0; ; i++ {
+		if GrammarType(i).String() == fmt.Sprintf("Invalid(%d)", i) {
+			break
+		}
+	}
+	test.T(t, Token{IdentToken, []byte("data")}.String(), "Ident('data')")
 }
 
 func TestParseError(t *testing.T) {
@@ -166,9 +164,11 @@ func TestParseError(t *testing.T) {
 		css    string
 		col    int
 	}{
+		{false, "}", 2},
+		{true, "}", 1},
 		{false, "selector", 9},
-		{true, "color 0", 8},
-		{true, "--color 0", 10},
+		{true, "color 0", 7},
+		{true, "--color 0", 9},
 		{true, "--custom-variable:0", 0},
 	}
 	for _, tt := range parseErrorTests {
@@ -203,6 +203,17 @@ func TestReader(t *testing.T) {
 	}
 }
 
+func TestParseOffset(t *testing.T) {
+	p := NewParser(bytes.NewBufferString(`div{background:url(link);}`), false)
+	test.T(t, p.Offset(), 0)
+	_, _, _ = p.Next()
+	test.T(t, p.Offset(), 4) // div{
+	_, _, _ = p.Next()
+	test.T(t, p.Offset(), 25) // background:url(link);
+	_, _, _ = p.Next()
+	test.T(t, p.Offset(), 26) // }
+}
+
 ////////////////////////////////////////////////////////////////
 
 type Obj struct{}
