@@ -1,6 +1,8 @@
 package css
 
-import "github.com/tdewolff/parse/v2/buffer"
+import (
+	"github.com/tdewolff/parse/v2/buffer"
+)
 
 // IsIdent returns true if the bytes are a valid identifier.
 func IsIdent(b []byte) bool {
