@@ -9,25 +9,30 @@ import (
 )
 
 func TestError(t *testing.T) {
-	err := NewError("message", bytes.NewBufferString("buffer"), 3)
+	err := NewError(bytes.NewBufferString("buffer"), 3, "message")
 
 	line, column, context := err.Position()
 	test.T(t, line, 1, "line")
 	test.T(t, column, 4, "column")
 	test.T(t, "\n"+context, "\n    1: buffer\n          ^", "context")
 
-	test.T(t, err.Error(), "parse error:1:4: message\n    1: buffer\n          ^", "error")
+	test.T(t, err.Error(), "message on line 1 and column 4\n    1: buffer\n          ^", "error")
 }
 
 func TestErrorLexer(t *testing.T) {
 	l := buffer.NewLexer(bytes.NewBufferString("buffer"))
 	l.Move(3)
-	err := NewErrorLexer("message", l)
+	err := NewErrorLexer(l, "message")
 
 	line, column, context := err.Position()
 	test.T(t, line, 1, "line")
 	test.T(t, column, 4, "column")
 	test.T(t, "\n"+context, "\n    1: buffer\n          ^", "context")
 
-	test.T(t, err.Error(), "parse error:1:4: message\n    1: buffer\n          ^", "error")
+	test.T(t, err.Error(), "message on line 1 and column 4\n    1: buffer\n          ^", "error")
+}
+
+func TestErrorMessages(t *testing.T) {
+	err := NewError(bytes.NewBufferString("buffer"), 3, "message %d", 5)
+	test.T(t, err.Error(), "message 5 on line 1 and column 4\n    1: buffer\n          ^", "error")
 }
