@@ -1,4 +1,4 @@
-package html // import "github.com/tdewolff/parse/html"
+package html
 
 import (
 	"bytes"
@@ -44,7 +44,7 @@ func TestTokens(t *testing.T) {
 		{"<script><!--var x='<script>';--></script>", TTs{StartTagToken, StartTagCloseToken, TextToken, EndTagToken}},
 		{"<![CDATA[ test ]]>", TTs{TextToken}},
 		{"<svg>text</svg>", TTs{SvgToken}},
-		{"<math>text</math>", TTs{MathToken}},
+		{"<math>text</math gibberish>", TTs{MathToken}},
 		{`<svg>text<x a="</svg>"></x></svg>`, TTs{SvgToken}},
 		{"<a><svg>text</svg></a>", TTs{StartTagToken, StartTagCloseToken, SvgToken, EndTagToken}},
 
@@ -100,7 +100,12 @@ func TestTokens(t *testing.T) {
 		})
 	}
 
-	test.T(t, TokenType(100).String(), "Invalid(100)")
+	// coverage
+	for i := 0; ; i++ {
+		if TokenType(i).String() == fmt.Sprintf("Invalid(%d)", i) {
+			break
+		}
+	}
 }
 
 func TestTags(t *testing.T) {
@@ -207,6 +212,84 @@ func TestErrors(t *testing.T) {
 	}
 }
 
+func TestTextAndAttrVal(t *testing.T) {
+	l := NewLexer(bytes.NewBufferString(`<div attr="val" >text<!--comment--><!DOCTYPE doctype><![CDATA[cdata]]><script>js</script><svg>image</svg>`))
+	_, data := l.Next()
+	test.Bytes(t, data, []byte("<div"))
+	test.Bytes(t, l.Text(), []byte("div"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte(` attr="val"`))
+	test.Bytes(t, l.Text(), []byte("attr"))
+	test.Bytes(t, l.AttrVal(), []byte(`"val"`))
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte(">"))
+	test.Bytes(t, l.Text(), nil)
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("text"))
+	test.Bytes(t, l.Text(), []byte("text"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<!--comment-->"))
+	test.Bytes(t, l.Text(), []byte("comment"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<!DOCTYPE doctype>"))
+	test.Bytes(t, l.Text(), []byte(" doctype"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<![CDATA[cdata]]>"))
+	test.Bytes(t, l.Text(), []byte("cdata"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<script"))
+	test.Bytes(t, l.Text(), []byte("script"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte(">"))
+	test.Bytes(t, l.Text(), nil)
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("js"))
+	test.Bytes(t, l.Text(), nil)
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("</script>"))
+	test.Bytes(t, l.Text(), []byte("script"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<svg>image</svg>"))
+	test.Bytes(t, l.Text(), []byte("svg"))
+	test.Bytes(t, l.AttrVal(), nil)
+}
+
+func TestOffset(t *testing.T) {
+	l := NewLexer(bytes.NewBufferString(`<div attr="val">text</div>`))
+	test.T(t, l.Offset(), 0)
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 4) // <div
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 15) // attr="val"
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 16) // >
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 20) // text
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 26) // </div>
+}
+
 ////////////////////////////////////////////////////////////////
 
 var J int
