@@ -13,20 +13,17 @@ func TestEscapeAttrVal(t *testing.T) {
 	}{
 		{`xyz`, `xyz`},
 		{``, ``},
-		{`x&amp;z`, `x&amp;z`},
 		{`x/z`, `x/z`},
 		{`x'z`, `"x'z"`},
 		{`x"z`, `'x"z'`},
 		{`'x"z'`, `'x"z'`},
-		{`'x&#39;"&#39;z'`, `"x'&#34;'z"`},
-		{`"x&#34;'&#34;z"`, `'x"&#39;"z'`},
-		{`"x&#x27;z"`, `"x'z"`},
-		{`'x&#x00022;z'`, `'x"z'`},
-		{`'x"&gt;'`, `'x"&gt;'`},
-		{`You&#039;re encouraged to log in; however, it&#039;s not mandatory. [o]`, `"You're encouraged to log in; however, it's not mandatory. [o]"`},
+		{`'x'"'z'`, `"x'&#34;'z"`},
+		{`"x"'"z"`, `'x"&#39;"z'`},
+		{`"x'z"`, `"x'z"`},
+		{`'x'z'`, `"x'z"`},
 		{`a'b=""`, `'a&#39;b=""'`},
 		{`x<z`, `"x<z"`},
-		{`'x"&#39;"z'`, `'x"&#39;"z'`},
+		{`'x"'"z'`, `'x"&#39;"z'`},
 	}
 	var buf []byte
 	for _, tt := range escapeAttrValTests {
