@@ -1,4 +1,4 @@
-package js // import "github.com/tdewolff/parse/js"
+package js
 
 import (
 	"bytes"
@@ -59,14 +59,23 @@ func TestTokens(t *testing.T) {
 		{"`outer${{x: 10}}bar${ raw`nested${2}endnest` }end`", TTs{TemplateToken, PunctuatorToken, IdentifierToken, PunctuatorToken, NumericToken, PunctuatorToken, TemplateToken, IdentifierToken, TemplateToken, NumericToken, TemplateToken, TemplateToken}},
 
 		// early endings
-		{"'string", TTs{StringToken}},
+		{"'string", TTs{UnknownToken, IdentifierToken}},
 		{"'\n '\u2028", TTs{UnknownToken, LineTerminatorToken, UnknownToken, LineTerminatorToken}},
 		{"'str\\\U00100000ing\\0'", TTs{StringToken}},
 		{"'strin\\00g'", TTs{StringToken}},
 		{"/*comment", TTs{SingleLineCommentToken}},
-		{"a=/regexp", TTs{IdentifierToken, PunctuatorToken, RegexpToken}},
+		{"a=/regexp", TTs{IdentifierToken, PunctuatorToken, PunctuatorToken, IdentifierToken}},
 		{"\\u002", TTs{UnknownToken, IdentifierToken}},
 
+		// null characters
+		{"'string\x00'return", TTs{StringToken, IdentifierToken}},
+		{"//comment\x00comment\nreturn", TTs{SingleLineCommentToken, LineTerminatorToken, IdentifierToken}},
+		{"/*comment\x00*/return", TTs{SingleLineCommentToken, IdentifierToken}},
+		{"a=/regexp\x00/;return", TTs{IdentifierToken, PunctuatorToken, RegexpToken, PunctuatorToken, IdentifierToken}},
+		{"a=/regexp\\\x00/;return", TTs{IdentifierToken, PunctuatorToken, RegexpToken, PunctuatorToken, IdentifierToken}},
+		{"`template\x00`return", TTs{TemplateToken, IdentifierToken}},
+		{"`template\\\x00`return", TTs{TemplateToken, IdentifierToken}},
+
 		// coverage
 		{"Ø a〉", TTs{IdentifierToken, IdentifierToken, UnknownToken}},
 		{"0xg 0.f", TTs{NumericToken, IdentifierToken, NumericToken, PunctuatorToken, IdentifierToken}},
@@ -77,7 +86,7 @@ func TestTokens(t *testing.T) {
 		{"\\u{0029FEF}ident", TTs{IdentifierToken}},
 		{"\\u{}", TTs{UnknownToken, IdentifierToken, PunctuatorToken, PunctuatorToken}},
 		{"\\ugident", TTs{UnknownToken, IdentifierToken}},
-		{"'str\u2028ing'", TTs{UnknownToken, IdentifierToken, LineTerminatorToken, IdentifierToken, StringToken}},
+		{"'str\u2028ing'", TTs{UnknownToken, IdentifierToken, LineTerminatorToken, IdentifierToken, UnknownToken}},
 		{"a=/\\\n", TTs{IdentifierToken, PunctuatorToken, PunctuatorToken, UnknownToken, LineTerminatorToken}},
 		{"a=/x/\u200C\u3009", TTs{IdentifierToken, PunctuatorToken, RegexpToken, UnknownToken}},
 		{"a=/x\n", TTs{IdentifierToken, PunctuatorToken, PunctuatorToken, IdentifierToken, LineTerminatorToken}},
@@ -134,8 +143,29 @@ func TestTokens(t *testing.T) {
 		})
 	}
 
-	test.T(t, WhitespaceToken.String(), "Whitespace")
-	test.T(t, TokenType(100).String(), "Invalid(100)")
+	// coverage
+	for i := 0; ; i++ {
+		if TokenType(i).String() == fmt.Sprintf("Invalid(%d)", i) {
+			break
+		}
+	}
+}
+
+func TestOffset(t *testing.T) {
+	l := NewLexer(bytes.NewBufferString(`var i=5;`))
+	test.T(t, l.Offset(), 0)
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 3) // var
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 4) // ws
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 5) // i
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 6) // =
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 7) // 5
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 8) // ;
 }
 
 ////////////////////////////////////////////////////////////////
