@@ -1,4 +1,4 @@
-package json // import "github.com/tdewolff/parse/json"
+package json
 
 import (
 	"bytes"
@@ -35,7 +35,6 @@ func TestGrammars(t *testing.T) {
 		{`{"a": "b", "c": "d"}`, GTs{StartObjectGrammar, StringGrammar, StringGrammar, StringGrammar, StringGrammar, EndObjectGrammar}},
 		{`{"a": [1, 2], "b": {"c": 3}}`, GTs{StartObjectGrammar, StringGrammar, StartArrayGrammar, NumberGrammar, NumberGrammar, EndArrayGrammar, StringGrammar, StartObjectGrammar, StringGrammar, NumberGrammar, EndObjectGrammar, EndObjectGrammar}},
 		{"[null,]", GTs{StartArrayGrammar, LiteralGrammar, EndArrayGrammar}},
-		// {"[\"x\\\x00y\", 0]", GTs{StartArrayGrammar, StringGrammar, NumberGrammar, EndArrayGrammar}},
 	}
 	for _, tt := range grammarTests {
 		t.Run(tt.json, func(t *testing.T) {
@@ -59,13 +58,38 @@ func TestGrammars(t *testing.T) {
 		})
 	}
 
-	test.T(t, WhitespaceGrammar.String(), "Whitespace")
-	test.T(t, GrammarType(100).String(), "Invalid(100)")
-	test.T(t, ValueState.String(), "Value")
-	test.T(t, ObjectKeyState.String(), "ObjectKey")
-	test.T(t, ObjectValueState.String(), "ObjectValue")
-	test.T(t, ArrayState.String(), "Array")
-	test.T(t, State(100).String(), "Invalid(100)")
+	// coverage
+	for i := 0; ; i++ {
+		if GrammarType(i).String() == fmt.Sprintf("Invalid(%d)", i) {
+			break
+		}
+	}
+	for i := 0; ; i++ {
+		if State(i).String() == fmt.Sprintf("Invalid(%d)", i) {
+			break
+		}
+	}
+}
+
+func TestGrammarsErrorEOF(t *testing.T) {
+	var grammarErrorTests = []struct {
+		json string
+	}{
+		{`{"":"`},
+		{"\"a\\"},
+	}
+	for _, tt := range grammarErrorTests {
+		t.Run(tt.json, func(t *testing.T) {
+			p := NewParser(bytes.NewBufferString(tt.json))
+			for {
+				grammar, _ := p.Next()
+				if grammar == ErrorGrammar {
+					test.T(t, p.Err(), io.EOF)
+					break
+				}
+			}
+		})
+	}
 }
 
 func TestGrammarsError(t *testing.T) {
@@ -81,8 +105,12 @@ func TestGrammarsError(t *testing.T) {
 		{"{\"a\" 1}", 6},
 		{"1.", 2},
 		{"1e+", 2},
-		{`{"":"`, 0},
-		{"\"a\\", 0},
+		{"[true, \x00]", 8},
+		{"\"string\x00\"", 8},
+		{"{\"id\": noquote}", 8},
+		{"{\"id\"\x00: 5}", 6},
+		{"{\"id: \x00}", 7},
+		{"{\"id: 5\x00", 8},
 	}
 	for _, tt := range grammarErrorTests {
 		t.Run(tt.json, func(t *testing.T) {
@@ -90,13 +118,11 @@ func TestGrammarsError(t *testing.T) {
 			for {
 				grammar, _ := p.Next()
 				if grammar == ErrorGrammar {
-					if tt.col == 0 {
-						test.T(t, p.Err(), io.EOF)
-					} else if perr, ok := p.Err().(*parse.Error); ok {
+					if perr, ok := p.Err().(*parse.Error); ok {
 						_, col, _ := perr.Position()
 						test.T(t, col, tt.col)
 					} else {
-						test.Fail(t, "bad error:", p.Err())
+						test.Fail(t, "not a parse error:", p.Err())
 					}
 					break
 				}
@@ -138,6 +164,29 @@ func TestStates(t *testing.T) {
 	}
 }
 
+func TestOffset(t *testing.T) {
+	p := NewParser(bytes.NewBufferString(`{"key": [5, "string", null, true]}`))
+	test.T(t, p.Offset(), 0)
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 1) // {
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 7) // "key":
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 9) // [
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 10) // 5
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 20) // , "string"
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 26) // , null
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 32) // , true
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 33) // ]
+	_, _ = p.Next()
+	test.T(t, p.Offset(), 34) // }
+}
+
 ////////////////////////////////////////////////////////////////
 
 func ExampleNewParser() {
