@@ -1,6 +1,7 @@
-package strconv // import "github.com/tdewolff/parse/strconv"
+package strconv
 
 import (
+	"fmt"
 	"testing"
 
 	"github.com/tdewolff/test"
@@ -21,9 +22,16 @@ func TestAppendPrice(t *testing.T) {
 		{123456789012, true, "1,234,567,890.12"},
 		{9223372036854775807, true, "92,233,720,368,547,758.07"},
 		{-9223372036854775808, true, "92,233,720,368,547,758.08"},
+		{149, false, "1"},
+		{150, false, "2"},
 	}
+
 	for _, tt := range priceTests {
-		price := AppendPrice([]byte{}, tt.price, tt.dec, ',', '.')
-		test.String(t, string(price), tt.expected, "for", tt.price)
+		t.Run(fmt.Sprint(tt.price), func(t *testing.T) {
+			price := AppendPrice(make([]byte, 0, 4), tt.price, tt.dec, ',', '.')
+			test.String(t, string(price), tt.expected, "for", tt.price)
+		})
 	}
+
+	// coverage
 }
