@@ -92,6 +92,21 @@ func (l *Lexer) Restore() {
 	l.r.Restore()
 }
 
+// Offset returns the current position in the input stream.
+func (l *Lexer) Offset() int {
+	return l.r.Offset()
+}
+
+// Text returns the textual representation of a token. This excludes delimiters and additional leading/trailing characters.
+func (l *Lexer) Text() []byte {
+	return l.text
+}
+
+// AttrVal returns the attribute value when an AttributeToken was returned from Next.
+func (l *Lexer) AttrVal() []byte {
+	return l.attrVal
+}
+
 // Next returns the next Token. It returns ErrorToken when an error was encountered. Using Err() one can retrieve the error message.
 func (l *Lexer) Next() (TokenType, []byte) {
 	l.text = nil
@@ -107,25 +122,22 @@ func (l *Lexer) Next() (TokenType, []byte) {
 		}
 		if c == 0 {
 			if l.r.Err() == nil {
-				l.err = parse.NewErrorLexer("unexpected null character", l.r)
+				l.err = parse.NewErrorLexer(l.r, "XML parse error: unexpected NULL character")
 			}
 			return ErrorToken, nil
 		} else if c != '>' && (c != '/' && c != '?' || l.r.Peek(1) != '>') {
 			return AttributeToken, l.shiftAttribute()
 		}
-		start := l.r.Pos()
+		l.r.Skip()
 		l.inTag = false
 		if c == '/' {
 			l.r.Move(2)
-			l.text = l.r.Lexeme()[start:]
 			return StartTagCloseVoidToken, l.r.Shift()
 		} else if c == '?' {
 			l.r.Move(2)
-			l.text = l.r.Lexeme()[start:]
 			return StartTagClosePIToken, l.r.Shift()
 		} else {
 			l.r.Move(1)
-			l.text = l.r.Lexeme()[start:]
 			return StartTagCloseToken, l.r.Shift()
 		}
 	}
@@ -134,7 +146,8 @@ func (l *Lexer) Next() (TokenType, []byte) {
 		c = l.r.Peek(0)
 		if c == '<' {
 			if l.r.Pos() > 0 {
-				return TextToken, l.r.Shift()
+				l.text = l.r.Shift()
+				return TextToken, l.text
 			}
 			c = l.r.Peek(1)
 			if c == '/' {
@@ -163,10 +176,11 @@ func (l *Lexer) Next() (TokenType, []byte) {
 			return StartTagToken, l.shiftStartTag()
 		} else if c == 0 {
 			if l.r.Pos() > 0 {
-				return TextToken, l.r.Shift()
+				l.text = l.r.Shift()
+				return TextToken, l.text
 			}
 			if l.r.Err() == nil {
-				l.err = parse.NewErrorLexer("unexpected null character", l.r)
+				l.err = parse.NewErrorLexer(l.r, "XML parse error: unexpected NULL character")
 			}
 			return ErrorToken, nil
 		}
@@ -174,16 +188,6 @@ func (l *Lexer) Next() (TokenType, []byte) {
 	}
 }
 
-// Text returns the textual representation of a token. This excludes delimiters and additional leading/trailing characters.
-func (l *Lexer) Text() []byte {
-	return l.text
-}
-
-// AttrVal returns the attribute value when an AttributeToken was returned from Next.
-func (l *Lexer) AttrVal() []byte {
-	return l.attrVal
-}
-
 ////////////////////////////////////////////////////////////////
 
 // The following functions follow the specifications at http://www.w3.org/html/wg/drafts/html/master/syntax.html
