@@ -1,4 +1,4 @@
-package xml // import "github.com/tdewolff/parse/xml"
+package xml
 
 import (
 	"bytes"
@@ -76,7 +76,12 @@ func TestTokens(t *testing.T) {
 		})
 	}
 
-	test.T(t, TokenType(100).String(), "Invalid(100)")
+	// coverage
+	for i := 0; ; i++ {
+		if TokenType(i).String() == fmt.Sprintf("Invalid(%d)", i) {
+			break
+		}
+	}
 }
 
 func TestTags(t *testing.T) {
@@ -155,7 +160,20 @@ func TestErrors(t *testing.T) {
 		col int
 	}{
 		{"a\x00b", 2},
-		{"<a\x00>", 3},
+		{"<\x00 b='5'>", 2},
+		{"<a\x00b='5'>", 3},
+		{"<a \x00='5'>", 4},
+		{"<a b\x00'5'>", 5},
+		{"<a b=\x005'>", 6},
+		{"<a b='\x00'>", 7},
+		{"<a b='5\x00>", 8},
+		{"<a b='5'\x00", 9},
+		{"</\x00a>", 3},
+		{"</ \x00>", 4},
+		{"</ a\x00", 5},
+		{"<!\x00", 3},
+		{"<![CDATA[\x00", 10},
+		{"/*\x00", 3},
 	}
 	for _, tt := range errorTests {
 		t.Run(tt.xml, func(t *testing.T) {
@@ -163,9 +181,7 @@ func TestErrors(t *testing.T) {
 			for {
 				token, _ := l.Next()
 				if token == ErrorToken {
-					if tt.col == 0 {
-						test.T(t, l.Err(), io.EOF)
-					} else if perr, ok := l.Err().(*parse.Error); ok {
+					if perr, ok := l.Err().(*parse.Error); ok {
 						_, col, _ := perr.Position()
 						test.T(t, col, tt.col)
 					} else {
@@ -178,6 +194,59 @@ func TestErrors(t *testing.T) {
 	}
 }
 
+func TestTextAndAttrVal(t *testing.T) {
+	l := NewLexer(bytes.NewBufferString(`<xml attr="val" >text<!--comment--><!DOCTYPE doctype><![CDATA[cdata]]>`))
+	_, data := l.Next()
+	test.Bytes(t, data, []byte("<xml"))
+	test.Bytes(t, l.Text(), []byte("xml"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte(` attr="val"`))
+	test.Bytes(t, l.Text(), []byte("attr"))
+	test.Bytes(t, l.AttrVal(), []byte(`"val"`))
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte(">"))
+	test.Bytes(t, l.Text(), nil)
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("text"))
+	test.Bytes(t, l.Text(), []byte("text"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<!--comment-->"))
+	test.Bytes(t, l.Text(), []byte("comment"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<!DOCTYPE doctype>"))
+	test.Bytes(t, l.Text(), []byte(" doctype"))
+	test.Bytes(t, l.AttrVal(), nil)
+
+	_, data = l.Next()
+	test.Bytes(t, data, []byte("<![CDATA[cdata]]>"))
+	test.Bytes(t, l.Text(), []byte("cdata"))
+	test.Bytes(t, l.AttrVal(), nil)
+}
+
+func TestOffset(t *testing.T) {
+	l := NewLexer(bytes.NewBufferString(`<div attr="val">text</div>`))
+	test.T(t, l.Offset(), 0)
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 4) // <div
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 15) // attr="val"
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 16) // >
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 20) // text
+	_, _ = l.Next()
+	test.T(t, l.Offset(), 26) // </div>
+}
+
 ////////////////////////////////////////////////////////////////
 
 func ExampleNewLexer() {
