@@ -11,15 +11,14 @@ func TestEscapeAttrVal(t *testing.T) {
 		attrVal  string
 		expected string
 	}{
-		{"xyz", "\"xyz\""},
-		{"", "\"\""},
-		{"x&amp;z", "\"x&amp;z\""},
-		{"x'z", "\"x'z\""},
-		{"x\"z", "'x\"z'"},
-		{"a'b=\"\"", "'a&#39;b=\"\"'"},
-		{"'x&#39;\"&#39;z'", "\"x'&#34;'z\""},
-		{"\"x&#34;'&#34;z\"", "'x\"&#39;\"z'"},
-		{"a&#39;b=\"\"", "'a&#39;b=\"\"'"},
+		{`xyz`, `"xyz"`},
+		{``, `""`},
+		{`x'z`, `"x'z"`},
+		{`x"z`, `'x"z'`},
+		{`a'b=""`, `'a&#39;b=""'`},
+		{`'x'"'z'`, `"x'&#34;'z"`},
+		{`"x"'"z"`, `'x"&#39;"z'`},
+		{`a'b=""`, `'a&#39;b=""'`},
 	}
 	var buf []byte
 	for _, tt := range attrValTests {
@@ -47,6 +46,7 @@ func TestEscapeCDATAVal(t *testing.T) {
 		{"<![CDATA[&&&&]]>", "<![CDATA[&&&&]]>"},
 		{"<![CDATA[ a ]]>", " a "},
 		{"<![CDATA[]]>", ""},
+		{"<![CDATA[ a ]]&gt; b ]]>", " a ]]&amp;gt; b "},
 	}
 	var buf []byte
 	for _, tt := range CDATAValTests {
